@@ -7,7 +7,6 @@ from typing import Any, Optional, TypeVar, cast
 import mediafile
 import pluggy
 from sqlalchemy import JSON, Column, Integer, String
-from sqlalchemy.ext.associationproxy import association_proxy
 from sqlalchemy.ext.mutable import MutableDict, MutableSet
 from sqlalchemy.orm import relationship
 from sqlalchemy.schema import ForeignKey, UniqueConstraint
@@ -90,7 +89,7 @@ class Hooks:
         See Also:
             * :ref:`Album and track fields <fields:Fields>`
             * `Mediafile docs <https://mediafile.readthedocs.io/en/latest/>`_
-            * The :meth:`~moe.plugins.write.Hooks.write_custom_tags` hook for writing
+            * The :meth:`~moe.write.Hooks.write_custom_tags` hook for writing
               tags.
         """
 
@@ -149,9 +148,10 @@ class MetaTrack(MetaLibItem):
     instance.
 
     Attributes:
-        album_obj (Optional[Album]): Corresponding Album object.
+        album (Optional[Album]): Corresponding Album object.
         artist (Optional[str])
         artists (Optional[set[str]]): Set of all artists.
+        custom (dict[str, Any]): Dictionary of custom fields.
         disc (Optional[int]): Disc number the track is on.
         genres (Optional[set[str]]): Set of all genres.
         title (Optional[str])
@@ -170,22 +170,18 @@ class MetaTrack(MetaLibItem):
         **kwargs,
     ):
         """Creates a MetaTrack object with any additional custom fields as kwargs."""
-        self._custom_fields = self._get_default_custom_fields()
-        self._custom_fields_set = set(self._custom_fields)
+        self.custom = kwargs
 
-        self.album_obj = album
+        self.album = album
         album.tracks.append(self)
 
         self.track_num = track_num
-        self.artist = artist or self.album_obj.artist
+        self.artist = artist or self.album.artist
         self.artists = artists
         self.disc = disc
         self.genres = genres
         self.title = title
 
-        for key, value in kwargs.items():
-            setattr(self, key, value)
-
         log.debug(f"MetaTrack created. [track={self!r}]")
 
     @property
@@ -212,14 +208,14 @@ class MetaTrack(MetaLibItem):
     def fields(self) -> set[str]:
         """Returns any editable, track-specific fields."""
         return {
-            "album_obj",
+            "album",
             "artist",
             "artists",
             "disc",
             "genres",
             "title",
             "track_num",
-        }.union(set(self._custom_fields))
+        }
 
     def merge(self, other: "MetaTrack", overwrite: bool = False):
         """Merges another track into this one.
@@ -232,13 +228,20 @@ class MetaTrack(MetaLibItem):
             f"Merging tracks. [track_a={self!r}, track_b={other!r}, {overwrite=!r}]"
         )
 
-        omit_fields = {"album_obj"}
+        omit_fields = {"album"}
         for field in self.fields - omit_fields:
             other_value = getattr(other, field, None)
             self_value = getattr(self, field, None)
             if other_value and (overwrite or (not overwrite and not self_value)):
                 setattr(self, field, other_value)
 
+        for custom_field in self.custom:
+            other_value = other.custom.get(custom_field)
+            if other_value and (
+                overwrite or (not overwrite and not self.custom[custom_field])
+            ):
+                self.custom[custom_field] = other_value
+
         log.debug(
             f"Tracks merged. [track_a={self!r}, track_b={other!r}, {overwrite=!r}]"
         )
@@ -258,29 +261,29 @@ class MetaTrack(MetaLibItem):
 
     def __lt__(self, other) -> bool:
         """Sort based on album, then disc, then track number."""
-        if self.album_obj == other.album_obj:
+        if self.album == other.album:
             if self.disc == other.disc:
                 return self.track_num < other.track_num
 
             return self.disc < other.disc
 
-        return self.album_obj < other.album_obj
+        return self.album < other.album
 
     def __repr__(self):
         """Represents a Track using track-specific and relevant album fields."""
         field_reprs = []
-        omit_fields = {"album_obj"}
+        omit_fields = {"album"}
         for field in self.fields - omit_fields:
             if hasattr(self, field):
                 field_reprs.append(f"{field}={getattr(self, field)!r}")
         repr_str = (
-            f"{__class__.__name__}("
+            f"{type(self).__name__}("
             + ", ".join(field_reprs)
-            + f", album='{self.album_obj}'"
+            + f", album='{self.album}'"
         )
 
         custom_field_reprs = []
-        for custom_field, value in self._custom_fields.items():
+        for custom_field, value in self.custom.items():
             custom_field_reprs.append(f"{custom_field}={value}")
         if custom_field_reprs:
             repr_str += ", custom_fields=[" + ", ".join(custom_field_reprs) + "]"
@@ -292,24 +295,15 @@ class MetaTrack(MetaLibItem):
         """String representation of a track."""
         return f"{self.artist} - {self.title}"
 
-    def _get_default_custom_fields(self) -> dict[str, Any]:
-        """Returns the default custom track fields."""
-        return {
-            field: default_val
-            for plugin_fields in config.CONFIG.pm.hook.create_custom_track_fields()
-            for field, default_val in plugin_fields.items()
-        }
-
 
 class Track(LibItem, SABase, MetaTrack):
     """A single track in the library.
 
     Attributes:
-        album (str)
-        albumartist (str)
-        album_obj (Album): Corresponding Album object.
+        album (Album): Corresponding Album object.
         artist (str)
         artists (Optional[set[str]]): Set of all artists.
+        custom (dict[str, Any]): Dictionary of custom fields.
         disc (int): Disc number the track is on.
         genres (Optional[set[str]]): Set of all genres.
         path (Path): Filesystem path of the track file.
@@ -333,7 +327,7 @@ class Track(LibItem, SABase, MetaTrack):
     )
     title: str = cast(str, Column(String, nullable=False))
     track_num: int = cast(int, Column(Integer, nullable=False))
-    _custom_fields: dict[str, Any] = cast(
+    custom: dict[str, Any] = cast(
         dict[str, Any],
         Column(
             MutableDict.as_mutable(JSON(none_as_null=True)),
@@ -343,9 +337,7 @@ class Track(LibItem, SABase, MetaTrack):
     )
 
     _album_id: int = cast(int, Column(Integer, ForeignKey("album._id")))
-    album_obj: Album = relationship("Album", back_populates="tracks")
-    album: str = association_proxy("album_obj", "title")
-    albumartist: str = association_proxy("album_obj", "artist")
+    album: Album = relationship("Album", back_populates="tracks")
 
     __table_args__ = (UniqueConstraint("disc", "track_num", "_album_id"),)
 
@@ -355,6 +347,10 @@ class Track(LibItem, SABase, MetaTrack):
         path: Path,
         title: str,
         track_num: int,
+        artist: Optional[str] = None,
+        artists: Optional[set[str]] = None,
+        disc: Optional[int] = None,
+        genres: Optional[set[str]] = None,
         **kwargs,
     ):
         """Creates a Track.
@@ -364,42 +360,42 @@ class Track(LibItem, SABase, MetaTrack):
             path: Filesystem path of the track file.
             title: Title of the track.
             track_num: Track number.
-            **kwargs: Any other fields to assign to the track.
+            artist: Track artist. Defaults to the album artist if not given.
+            artists: Set of all artists.
+            disc: Disc the track belongs to. If not given, will try to guess the disc
+                based on the ``path`` of the track.
+            genres (Optional[set[str]]): Set of all genres.
+            **kwargs: Any custom fields to assign to the track.
         """
-        self._custom_fields = self._get_default_custom_fields()
-        self._custom_fields_set = set(self._custom_fields)
+        self.custom = kwargs
 
         album.tracks.append(self)
+
         self.path = path
+        self.artist = artist or self.album.artist
+        self.artists = artists
+        self.disc = disc or self._guess_disc()
+        self.genres = genres
         self.title = title
         self.track_num = track_num
 
-        for key, value in kwargs.items():
-            setattr(self, key, value)
-
-        if self.artist is None:
-            self.artist = self.albumartist
-
-        if not self.disc:
-            self.disc = self._guess_disc()
-
         log.debug(f"Track created. [track={self!r}]")
 
     def _guess_disc(self) -> int:
         """Attempts to guess the disc of a track based on it's path."""
         log.debug(f"Guessing track disc number. [track={self!r}]")
 
-        if self.path.parent == self.album_obj.path:
+        if self.path.parent == self.album.path:
             return 1
 
         # The track is in a subdirectory of the album - most likely disc directories.
         disc_dirs: list[Path] = []
-        for path in self.album_obj.path.iterdir():
+        for path in self.album.path.iterdir():
             if not path.is_dir():
                 continue
 
             contains_tracks = False
-            for album_track in self.album_obj.tracks:
+            for album_track in self.album.tracks:
                 if album_track.path.is_relative_to(path):
                     contains_tracks = True
 
@@ -527,7 +523,7 @@ class Track(LibItem, SABase, MetaTrack):
         if (
             self.track_num == other.track_num
             and self.disc == other.disc
-            and self.album_obj == other.album_obj
+            and self.album == other.album
         ):
             return False
 
